From 5dd9c5b54d32af1023eb2ea84d8ad702cae3307e Mon Sep 17 00:00:00 2001
From: Eric Danan <eric.danan@u-cergy.fr>
Date: Fri, 8 Jun 2018 16:26:21 +0200
Subject: [PATCH] org-capture.el: Offer property value completions in capture
 template

* lisp/org-capture.el (org-capture-fill-template): When reading a
  property's value for a `%^{prop}p' template element, offer
  completions on the property's allowed or existing values.

This is done by temporarily prepending to the variable
`org-file-property' and entry `(prop_all . vals)' where vals is either
the local or inherited value of `prop_all' at the target location or,
if that is nil, the set of local values of `prop' in the target
buffer.  A dummy value `:ETC' is added in the latter case so that
matching is not required.

The resulting behavior is (hopefully) the same as when using
`org-set-property' directly.  If there is an entry for `prop' in
`org-property-allowed-value-functions', that will be used in
priority.  If not and there is a local or inherited value for
`prop_all' at point or target location, then this will be used.  In
these two cases, matching is required unless `:ETC' belongs to the
allowed values.  Finally, in the absence of allowed values, the set of
local values for `prop' in the current or target buffer is used, with
matching not required.
---
 lisp/org-capture.el | 10 +++++++++-
 1 file changed, 9 insertions(+), 1 deletion(-)

diff --git a/lisp/org-capture.el b/lisp/org-capture.el
index eeddd2ab..b10ba576 100644
--- a/lisp/org-capture.el
+++ b/lisp/org-capture.el
@@ -1695,7 +1695,15 @@ (defun org-capture-fill-template (&optional template initial annotation)
 						first-value)))
 			 (_ (error "Invalid `org-capture--clipboards' value: %S"
 				   org-capture--clipboards)))))
-		    ("p" (org-set-property prompt nil))
+		    ("p"
+		     (let* ((prop-all (concat prompt "_ALL"))
+			    (vals (with-current-buffer buffer
+				    (or (org-entry-get nil prop-all 'inherit)
+					(concat (mapconcat 'identity (org-property-values prompt) " ")
+						" :ETC"))))
+			    (org-file-properties (cons (cons prop-all vals)
+						       org-file-properties)))
+		       (org-set-property prompt nil)))
 		    ((or "t" "T" "u" "U")
 		     ;; These are the date/time related ones.
 		     (let* ((upcase? (equal (upcase key) key))
-- 
2.17.0

